# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,safe_call)
S3method(as.data.frame,symbol_registry)
S3method(audit,ArmadilloCredentials)
S3method(audit,character)
S3method(audit,cr8tor)
S3method(audit,default)
S3method(audit,list)
S3method(audit,opal)
S3method(audit,rocrate)
S3method(audit_engine,cr8tor)
S3method(audit_engine,default)
S3method(audit_engine,opal)
S3method(backend_logs,opal)
S3method(backend_options,opal)
S3method(backend_packages,opal)
S3method(backend_profile_exists,opal)
S3method(backend_project,opal)
S3method(backend_project_exists,opal)
S3method(backend_project_perms,opal)
S3method(backend_projects,opal)
S3method(backend_resource_perms,opal)
S3method(backend_resources,opal)
S3method(backend_sys_perms,opal)
S3method(backend_table_perms,opal)
S3method(backend_tables,opal)
S3method(backend_user_exists,opal)
S3method(backend_users,opal)
S3method(check_permissions,ArmadilloCredentials)
S3method(check_permissions,default)
S3method(check_permissions,opal)
S3method(extract_safe_data,opal)
S3method(extract_safe_data,rocrate)
S3method(extract_safe_output,opal)
S3method(extract_safe_output,rocrate)
S3method(extract_safe_people,opal)
S3method(extract_safe_people,rocrate)
S3method(extract_safe_project,opal)
S3method(extract_safe_project,rocrate)
S3method(extract_safe_setting,opal)
S3method(extract_safe_setting,rocrate)
S3method(filter_safe_people,opal)
S3method(flatten_safe_data,default)
S3method(flatten_safe_data,rocrate)
S3method(flatten_safe_output,default)
S3method(flatten_safe_output,rocrate)
S3method(flatten_safe_people,default)
S3method(flatten_safe_people,rocrate)
S3method(flatten_safe_project,default)
S3method(flatten_safe_project,rocrate)
S3method(flatten_safe_setting,default)
S3method(flatten_safe_setting,rocrate)
S3method(init,ArmadilloCredentials)
S3method(init,opal)
S3method(init,rocrate)
S3method(is_admin_con,opal)
S3method(is_audit_con,opal)
S3method(parse_user_profiles,ArmadilloCredentials)
S3method(parse_user_profiles,opal)
S3method(print,cr8tor_bundle)
S3method(print,safe_call)
S3method(print,safe_symbol)
S3method(project_exists,ArmadilloCredentials)
S3method(project_exists,opal)
S3method(report,ArmadilloCredentials)
S3method(report,character)
S3method(report,default)
S3method(report,list)
S3method(report,rocrate)
S3method(safe_call,call)
S3method(safe_call,character)
S3method(safe_data,ArmadilloCredentials)
S3method(safe_data,character)
S3method(safe_data,default)
S3method(safe_data,opal)
S3method(safe_data,rocrate)
S3method(safe_output,ArmadilloCredentials)
S3method(safe_output,character)
S3method(safe_output,default)
S3method(safe_output,opal)
S3method(safe_output,rocrate)
S3method(safe_people,ArmadilloCredentials)
S3method(safe_people,character)
S3method(safe_people,default)
S3method(safe_people,opal)
S3method(safe_people,rocrate)
S3method(safe_project,ArmadilloCredentials)
S3method(safe_project,character)
S3method(safe_project,default)
S3method(safe_project,opal)
S3method(safe_project,rocrate)
S3method(safe_setting,ArmadilloCredentials)
S3method(safe_setting,character)
S3method(safe_setting,cr8tor)
S3method(safe_setting,default)
S3method(safe_setting,opal)
S3method(safe_setting,rocrate)
S3method(validate_backend_version,default)
S3method(validate_backend_version,opal)
S3method(validate_con,default)
S3method(validate_con,opal)
export(armadillo_login)
export(audit)
export(check_permissions)
export(init)
export(report)
export(safe_data)
export(safe_output)
export(safe_people)
export(safe_project)
export(safe_setting)
importFrom(rocrateR,get_entity)
importFrom(utils,write.csv)
