% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-safe_data.R
\name{flatten_safe_data}
\alias{flatten_safe_data}
\alias{flatten_safe_data.default}
\alias{flatten_safe_data.rocrate}
\title{Flatten object with Safe Data details}
\usage{
flatten_safe_data(x, ...)

\method{flatten_safe_data}{default}(x, ...)

\method{flatten_safe_data}{rocrate}(x, ..., id = NULL, asset_id_suffix = "#asset:")
}
\arguments{
\item{x}{Object (e.g., RO-Crate) with Safe Data details. This can be
generated with the \code{\link[=extract_safe_data]{extract_safe_data()}} function.}

\item{id}{Vector of strings with the \verb{@id}s for the datasets to be extracted.
If not provided, extract all entities with \verb{@type = 'Dataset'}.}
}
\value{
Data frame with Safe Data details.
}
\description{
Flatten object with Safe Data details
}
\keyword{internal}
