% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-safe_setting.R
\name{extract_safe_setting}
\alias{extract_safe_setting}
\alias{extract_safe_setting.opal}
\alias{extract_safe_setting.rocrate}
\title{Extract Safe Setting entity(ies)}
\usage{
extract_safe_setting(x, ...)

\method{extract_safe_setting}{opal}(x, ..., rocrate = rocrateR::rocrate_5s())

\method{extract_safe_setting}{rocrate}(x, ..., id = NULL, rocrate = rocrateR::rocrate_5s())
}
\arguments{
\item{x}{This can be a connection to a 'DataSHIELD' server (e.g., object with
the \code{opal} class, see \code{\link[opalr:opal.login]{opalr::opal.login()}}), an RO-Crate
(\link[rocrateR:rocrate]{rocrate} class) or a string with the path to an
RO-Crate.}

\item{...}{Other optional arguments. See the full documentation}

\item{rocrate}{(Optional) RO-Crate object to update with Safe Setting details.}

\item{id}{(Optional) Vector with \verb{@id} strings for Safe Setting entity(ies)
to be extracted from the given RO-Crate, \code{x}.}
}
\value{
RO-Crate with Safe Setting entity(ies).
}
\description{
Extract Safe Setting entity(ies)
}
\keyword{internal}
