% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-opal.R
\name{get_table_permissions}
\alias{get_table_permissions}
\title{Get table permissions}
\usage{
get_table_permissions(x, project, tables)
}
\arguments{
\item{x}{Connection to OBiBa's Opal server (see \code{\link[opalr:opal.login]{opalr::opal.login()}}).}

\item{project}{String with project name.}

\item{tables}{String (or vector of strings) with table names for the given
project.}
}
\value{
Data frame with permissions for each table in \code{tables}.
}
\description{
Wrapper for the \code{\link[opalr:opal.table_perm]{opalr::opal.table_perm()}} function.
}
\seealso{
Other Opal: 
\code{\link{get_project_details}()},
\code{\link{get_project_tables}()},
\code{\link{parse_user_profiles}()},
\code{\link{project_exists}()}
}
\concept{Opal}
\keyword{internal}
