% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-safe_output.R
\name{extract_safe_output}
\alias{extract_safe_output}
\alias{extract_safe_output.opal}
\alias{extract_safe_output.rocrate}
\title{Extract Safe Output entity(ies)}
\usage{
extract_safe_output(x, ...)

\method{extract_safe_output}{opal}(
  x,
  ...,
  path = NULL,
  user = NULL,
  logs_to = Sys.time(),
  logs_from = logs_to - 24 * 60^2,
  rocrate = rocrateR::rocrate_5s()
)

\method{extract_safe_output}{rocrate}(
  x,
  ...,
  id = NULL,
  user = NULL,
  rocrate = rocrateR::rocrate_5s()
)
}
\arguments{
\item{x}{This can be a connection to a 'DataSHIELD' server (e.g., object with
the \code{opal} class, see \code{\link[opalr:opal.login]{opalr::opal.login()}}), an RO-Crate
(\link[rocrateR:rocrate]{rocrate} class) or a string with the path to an
RO-Crate.}

\item{...}{Other optional arguments. See the full documentation}

\item{path}{String with path pointing to the root of the RO-Crate. This will
be used to store log files. If not provided, logs will be stored within
the RO-Crate returned by this function.}

\item{user}{List (or \link[rocrateR:entity]{entity} object) with details for
the Safe People, it must include \verb{@id} and \code{name} entries. Alternatively,
this can be a string with the \code{name} of the current user.}

\item{logs_to}{Upper limit timestamp to filter out the outputs generated
(default: \code{Sys.time()}, current system time).}

\item{logs_from}{Lower limit timestamp to filter out the outputs generated
(default: \code{Sys.time() - 24 * 60 ^ 2}, last 24 hours).}

\item{rocrate}{(Optional) RO-Crate object to update with Safe Output details.}

\item{id}{(Optional) Vector with \verb{@id} strings for Safe Output entity(ies)
to be extracted from the given RO-Crate, \code{x}.}
}
\value{
RO-Crate with Safe Output entity(ies).
}
\description{
Extract Safe Output entity(ies)
}
\keyword{internal}
