% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-safe_data.R
\name{extract_safe_data}
\alias{extract_safe_data}
\alias{extract_safe_data.opal}
\alias{extract_safe_data.rocrate}
\title{Extract Safe Data entity(ies)}
\usage{
extract_safe_data(x, ...)

\method{extract_safe_data}{opal}(x, ..., rocrate = rocrateR::rocrate_5s())

\method{extract_safe_data}{rocrate}(
  x,
  ...,
  id = NULL,
  asset_id_suffix = "#asset:",
  rocrate = rocrateR::rocrate_5s()
)
}
\arguments{
\item{x}{This can be a connection to a 'DataSHIELD' server (e.g., object with
the \code{opal} class, see \code{\link[opalr:opal.login]{opalr::opal.login()}}), an RO-Crate
(\link[rocrateR:rocrate]{rocrate} class) or a string with the path to an
RO-Crate.}

\item{...}{Other optional arguments. See the full documentation}

\item{rocrate}{(Optional) RO-Crate object to update with Safe Data details.}

\item{id}{(Optional) Vector with \verb{@id} strings for Safe Data entity(ies)
to be extracted from the given RO-Crate, \code{x}.}

\item{asset_id_suffix}{String with ID suffix for the tables/datasets
entities in the RO-Crate (default: \code{"#asset:"}).}
}
\value{
RO-Crate with Safe Data entity(ies).
}
\description{
Extract Safe Data entity(ies)
}
\keyword{internal}
