% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-tibble.R
\name{refill_vec}
\alias{refill_vec}
\title{Fill vector}
\usage{
refill_vec(x, val = "")
}
\arguments{
\item{x}{Input vector.}

\item{val}{Default value.}
}
\value{
Filled vector.
}
\description{
Fill vector
}
\keyword{internal}
