% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-opal.R
\name{user_perm_entity}
\alias{user_perm_entity}
\title{Create user/person permission entities}
\usage{
user_perm_entity(person, person_id, asset, asset_id, permission, ...)
}
\arguments{
\item{person}{String with person name/username.}

\item{person_id}{String with person \verb{@id}.}

\item{asset}{String with dataset/table/resource name.}

\item{asset_id}{String with dataset/table \verb{@id}.}

\item{permission}{String with permission ('view', 'view-values', 'edit',
'edit-values' OR 'administrate').}

\item{...}{Other additional values.}
}
\value{
List of \link[rocrateR:entity]{rocrateR::entity} objects
}
\description{
Create user/person permission entities
}
\keyword{internal}
