% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-opal.R
\name{flatten_user_perm_entity}
\alias{flatten_user_perm_entity}
\title{Flatten user permission entities}
\usage{
flatten_user_perm_entity(x)
}
\arguments{
\item{x}{List with entities, generated by \code{\link[=user_perm_entity]{user_perm_entity()}}.}
}
\value{
Tibble with properties of the entities.
}
\description{
Flatten user permission entities
}
\keyword{internal}
