% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-connection.R
\name{parse_user_profiles}
\alias{parse_user_profiles}
\alias{parse_user_profiles,armadillo-method}
\alias{parse_user_profiles.opal}
\alias{parse_user_profiles.ArmadilloCredentials}
\title{Parse user profiles}
\usage{
\S4method{parse_user_profiles}{armadillo}(x, ..., user)

\method{parse_user_profiles}{opal}(x, ..., user)

\method{parse_user_profiles}{ArmadilloCredentials}(x, ..., user)
}
\arguments{
\item{x}{Connection object to backend for DataSHIELD server (e.g., Opal).}

\item{...}{Optional arguments, unused.}
}
\value{
Data frame with given \code{user}'s profile details, as captured on the
server pointed by \code{x}.
}
\description{
Parse user profiles
}
\seealso{
Other Armadillo: 
\code{\link{project_exists}()}

Other Opal: 
\code{\link{get_project_details}()},
\code{\link{get_project_tables}()},
\code{\link{get_table_permissions}()},
\code{\link{project_exists}()}
}
\concept{Armadillo}
\concept{Opal}
\keyword{internal}
