% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-opal.R
\name{get_project_tables}
\alias{get_project_tables}
\title{Get project tables}
\usage{
get_project_tables(x, project)
}
\arguments{
\item{x}{Connection to OBiBa's Opal server (see \code{\link[opalr:opal.login]{opalr::opal.login()}}).}

\item{project}{String with project name.}
}
\value{
List of project tables
}
\description{
Wrapper for \code{\link[opalr:opal.project]{opalr::opal.project()}}.
}
\seealso{
Other Opal: 
\code{\link{get_project_details}()},
\code{\link{get_table_permissions}()},
\code{\link{parse_user_profiles}()},
\code{\link{project_exists}()}
}
\concept{Opal}
\keyword{internal}
