% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-opal.R
\name{is_opal_admin_con}
\alias{is_opal_admin_con}
\title{Verify if connection was created by an administrative user}
\usage{
is_opal_admin_con(x)
}
\arguments{
\item{x}{Connection to OBiBa's Opal server (see \code{\link[opalr:opal.login]{opalr::opal.login()}}).}
}
\value{
Boolean flag to indicate whether the given connection was created
by an administrative user.
}
\description{
Verify if connection was created by an administrative user
}
\keyword{internal}
