% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/safe_setting.R
\name{safe_setting}
\alias{safe_setting}
\alias{safe_setting.character}
\alias{safe_setting.cr8tor}
\alias{safe_setting.opal}
\alias{safe_setting.rocrate}
\title{Safe Setting details}
\source{
\itemize{
\item Research Data Scotland, 2025. "What is the Five Safes framework?".
\url{https://www.researchdata.scot/engage-and-learn/data-explainers/what-is-the-five-safes-framework/}
}
}
\usage{
safe_setting(x, ...)

\method{safe_setting}{character}(
  x,
  ...,
  connection = attr(x, "connection"),
  path = attr(x, "path"),
  profile = attr(x, "profile"),
  project = attr(x, "project"),
  tables = attr(x, "tables"),
  resources = attr(x, "resources"),
  user = attr(x, "user")
)

\method{safe_setting}{cr8tor}(x, ..., rocrate = rocrateR::rocrate_5s())

\method{safe_setting}{opal}(
  x,
  ...,
  rocrate = rocrateR::rocrate_5s(),
  path = NULL,
  profile = "default",
  project = NULL,
  resources = NULL,
  tables = NULL,
  user = NULL
)

\method{safe_setting}{rocrate}(
  x,
  ...,
  connection = attr(x, "connection"),
  path = attr(x, "path"),
  profile = attr(x, "profile"),
  project = attr(x, "project"),
  resources = attr(x, "resources"),
  tables = attr(x, "tables"),
  user = attr(x, "user")
)
}
\arguments{
\item{x}{This can be a connection to a 'DataSHIELD' server (e.g., object with
the \code{opal} class, see \code{\link[opalr:opal.login]{opalr::opal.login()}}), an RO-Crate
(\link[rocrateR:rocrate]{rocrate} class) or a string with the path to an
RO-Crate.}

\item{...}{Other optional arguments. See the full documentation,
\code{\link[=safe_setting]{?dsROCrate::safe_setting}}.}

\item{connection}{Connection object for the 'DataSHIELD' server where the
values will be extracted from (e.g., OBiBa's Opal). Optional, if \code{x} is
set to a connection object. If so, then \code{rocrate} is required.}

\item{path}{String with path pointing to the root of the RO-Crate. This will
be used to store log files. If not provided, logs will be stored within
the RO-Crate returned by this function.}

\item{profile}{String with profile name (used for OBiBa's Opal backend).}

\item{project}{String with the name of the \link[=safe_project]{Safe Project}.}

\item{tables}{Vector of strings with the names of the tables/datasets, part
of \code{project}. Optional, if not provided, all the tables/datasets
associated to \code{project} will be included in the RO-Crate.}

\item{resources}{Vector of strings with the names of the resources, part of
\code{project}. Optional, if not provided, all the resources associated to
\code{project} will be included in the RO-Crate.}

\item{user}{List (or \link[rocrateR:entity]{entity} object) with details for
the Safe People, it must include \verb{@id} and \code{name} entries. Alternatively,
this can be a string with the \code{name} of the current user.}

\item{rocrate}{RO-Crate object. Optional, if \code{x} is either an RO-Crate
object or a path to a valid RO-Crate. If so, then \code{connection} is
required (default: \code{rocrateR::rocrate_5s()}).}
}
\value{
Updated RO-Crate object with Safe Settings information.
}
\description{
Safe Setting details for the RO-Crate.
}
\details{
The organisational and technical settings used to access data are designed
to minimise the risk of accidental disclosure of data.

These settings also prevent the deliberate disclosure of data to others.

Physical settings for data access can include locations like
\href{https://safepodnetwork.ac.uk/}{SafePods} – secured rooms that use controlled
door access, CCTV and secure technology to ensure that sensitive data cannot
be mishandled or removed from the Safe Setting. Researchers can analyse the
data in these secure rooms, but do not have access to the internet, external
devices (such as printers), or any other way of removing protected data from
the space.

Digital Safe Settings provide secure access to data from a remote location.
To be approved for remote data access, researchers will need to prove that
their organisation meets physical and IT security standards.
}
