% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-opal.R
\name{update_project_datasets}
\alias{update_project_datasets}
\title{Update datasets linked to a project}
\usage{
update_project_datasets(rocrate, project, ds_ids)
}
\arguments{
\item{rocrate}{RO-Crate object. Optional, if \code{x} is either an RO-Crate
object or a path to a valid RO-Crate. If so, then \code{connection} is
required (default: \code{rocrateR::rocrate_5s()}).}

\item{project}{String with the name of the \link[=safe_project]{Safe Project}.}

\item{ds_ids}{Vector with \verb{@id}s of the datasets to be linked to \code{project}}
}
\value{
Update RO-Crate with updated project entity
}
\description{
Update datasets linked to a project (\code{hasPart})
}
\keyword{internal}
