% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-opal.R
\name{validate_opal_con}
\alias{validate_opal_con}
\title{Validate OBiBa's Opal connection}
\usage{
validate_opal_con(x)
}
\arguments{
\item{x}{Connection to OBiBa's Opal server (see \code{\link[opalr:opal.login]{opalr::opal.login()}}).}
}
\value{
Nothing, call for its side effect.
}
\description{
Validate OBiBa's Opal connection
}
\keyword{internal}
