% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-date.R
\name{is_valid_posixct}
\alias{is_valid_posixct}
\title{Validate POSIXct string}
\usage{
is_valid_posixct(x, tz = "UTC")
}
\arguments{
\item{x}{POSIXct string.}

\item{tz}{String with time zone,}
}
\value{
Boolean value to indicate if the given string is a valid POSIXct
string.
}
\description{
Validate POSIXct string
}
\keyword{internal}
