% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-armadillo.R
\name{armadillo_login}
\alias{armadillo_login}
\title{Login to a MOLGENIS' Armadillo server}
\usage{
armadillo_login(server)
}
\arguments{
\item{server}{the URL of the Armadillo server}
}
\value{
MOLGENIS' Armadillo connection object.
}
\description{
Login to a MOLGENIS' Armadillo server. Wrapper for the function
\code{\link[DSMolgenisArmadillo:armadillo.get_credentials]{DSMolgenisArmadillo::armadillo.get_credentials()}}.
}
