% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-safe_setting.R
\name{flatten_safe_setting}
\alias{flatten_safe_setting}
\alias{flatten_safe_setting.default}
\alias{flatten_safe_setting.rocrate}
\title{Flatten object with Safe Setting details}
\usage{
flatten_safe_setting(x, ...)

\method{flatten_safe_setting}{default}(x, ...)

\method{flatten_safe_setting}{rocrate}(x, ..., id = NULL)
}
\arguments{
\item{x}{Object (e.g., RO-Crate) with Safe Setting details. This can be
generated with the \code{\link[=extract_safe_setting]{extract_safe_setting()}} function.}

\item{id}{Vector of strings with the \verb{@id}s for the settings to be extracted.
If not provided, extract all entities with \verb{@type = 'PropertyValue'} or
\verb{@type = 'SoftwareApplication'}.}
}
\value{
Data frame with Safe Settings.
}
\description{
Flatten object with Safe Setting details
}
\keyword{internal}
