% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-tibble.R
\name{unfill_vec}
\alias{unfill_vec}
\title{Unfill vector}
\source{
https://github.com/tidyverse/tidyr/issues/250#issuecomment-344984802
}
\usage{
unfill_vec(x, val = "")
}
\arguments{
\item{x}{Input vector.}

\item{val}{Default value.}
}
\value{
Unfilled vector.
}
\description{
Unfill vector
}
\keyword{internal}
