% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/audit_engine.R
\name{audit_engine}
\alias{audit_engine}
\alias{audit_engine.cr8tor}
\alias{audit_engine.opal}
\title{Audit Engine}
\usage{
audit_engine(x, ...)

\method{audit_engine}{cr8tor}(x, ...)

\method{audit_engine}{opal}(
  x,
  ...,
  project = NULL,
  user = NULL,
  logs_from = -Inf,
  logs_to = Inf,
  path = NULL
)
}
\arguments{
\item{x}{This can be a connection to a 'DataSHIELD' server (e.g., object with
the \code{opal} class, see \code{\link[opalr:opal.login]{opalr::opal.login()}}). Alternatively, a governance
archive file, representing the intent of a project and associated
governance details.}

\item{...}{Other optional arguments, see full documentation for details.}

\item{project}{String with project name(s) from which to extra Safe Project
details.}

\item{user}{String with the user name for which to extract Safe People
details.}

\item{logs_from}{Lower limit timestamp to filter out the outputs generated
(default: \code{-Inf}, everything up to \code{logs_to})}

\item{logs_to}{Upper limit timestamp to filter out the outputs generated
(default: \code{Inf}, everything from \code{logs_from} onwards).}

\item{path}{String with path pointing to the root of the RO-Crate. This will
be used to store log files. If not provided, logs will be stored within
the RO-Crate returned by this function.}
}
\value{
Audit RO-Crate with 5 Safes Components.
}
\description{
Internal function to create audits for various back-ends.
}
\keyword{internal}
