% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-safe_project.R
\name{flatten_safe_project}
\alias{flatten_safe_project}
\alias{flatten_safe_project.default}
\alias{flatten_safe_project.rocrate}
\title{Flatten object with Safe Project details}
\usage{
flatten_safe_project(x, ...)

\method{flatten_safe_project}{default}(x, ...)

\method{flatten_safe_project}{rocrate}(x, ..., y = x)
}
\arguments{
\item{x}{Object (e.g., RO-Crate) with Safe Project details. This
can be generated with the \code{\link[=extract_safe_project]{extract_safe_project()}} function.}

\item{...}{Other optional arguments (not in used).}

\item{y}{Object (e.g., RO-Crate) with Safe Data details. This can be
generated with the \code{\link[=extract_safe_data]{extract_safe_data()}} function. If not provided, it
uses the \code{x} by default.}
}
\value{
Data frame with safe project details.
}
\description{
Flatten object with Safe Project details
}
\keyword{internal}
