% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{report}
\alias{report}
\alias{report.character}
\alias{report.default}
\alias{report.list}
\alias{report.rocrate}
\title{Create an RO-Crate report}
\usage{
report(x, ...)

\method{report}{character}(
  x,
  ...,
  title = "DataSHIELD Report",
  filepath = tempfile(fileext = ".md"),
  render = TRUE,
  doc_format = "html",
  overwrite = FALSE,
  include_user_perm = TRUE,
  diag_title = "DataSHIELD server",
  diag_width = NULL,
  diag_height = NULL,
  max_line_length = 200
)

\method{report}{default}(x, ...)

\method{report}{list}(
  x,
  ...,
  study_name,
  title = "DataSHIELD Report",
  filepath = tempfile(fileext = ".md"),
  render = TRUE,
  doc_format = "html",
  overwrite = FALSE,
  include_user_perm = TRUE,
  diag_title = "DataSHIELD server",
  diag_width = NULL,
  diag_height = NULL,
  max_line_length = 200
)

\method{report}{rocrate}(
  x,
  ...,
  title = "DataSHIELD Report",
  filepath = tempfile(fileext = ".md"),
  render = TRUE,
  doc_format = "html",
  overwrite = FALSE,
  include_user_perm = TRUE,
  diag_title = "DataSHIELD server",
  diag_width = NULL,
  diag_height = NULL,
  max_line_length = 200
)
}
\arguments{
\item{x}{This can be an RO-Crate (\link[rocrateR:rocrate]{rocrate} class) or a
string with the path to an RO-Crate.}

\item{...}{Other optional arguments. See the full documentation,
\code{\link[=report]{?dsROCrate::report}}.}

\item{title}{String with title for the report (default: 'DataSHIELD Report').}

\item{filepath}{String with file path for Markdown report with the summary
of the given object, \code{x}.}

\item{render}{Boolean flag to indicate whether to render the markdown report.}

\item{doc_format}{String with file format for the markdown report.}

\item{overwrite}{Boolean flag to indicate whether to overwrite a previous
version of markdown report.}

\item{include_user_perm}{Boolean flag to indicate whether to include user
permissions in the report overview's diagram.}

\item{diag_title}{String with title for the 'root' of the diagram (default:
'DataSHIELD server').}

\item{diag_width}{Numeric value with width (in inches) for the report
overview's diagram (default: \code{NULL}, estimated based on number of nodes).}

\item{diag_height}{Numeric value with height (in inches) for the report
overview's diagram (default: \code{NULL}, estimated based on number of nodes).}

\item{max_line_length}{Integer with the maximum number of characters per line
in the RO-Crate to be printed in the report.}

\item{study_name}{String with the study name.}
}
\value{
RO-Crate report as markdown (.md) file and/or HTML.
}
\description{
Create an RO-Crate report
}
