% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-rocrate.R
\name{load_content}
\alias{load_content}
\title{Load \code{content} from external files}
\usage{
load_content(rocrate, roc_path)
}
\arguments{
\item{rocrate}{Object with the \link[rocrateR:rocrate]{rocrate} class.}

\item{roc_path}{String with path to the root of the RO-Crate.}
}
\value{
Update \code{rocrate} object.
}
\description{
Load \code{content} from external files
}
\keyword{internal}
