% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-connection.R
\name{project_exists}
\alias{project_exists}
\alias{project_exists,armadillo-method}
\alias{project_exists.opal}
\alias{project_exists.ArmadilloCredentials}
\title{Verify if project exists}
\usage{
\S4method{project_exists}{armadillo}(
  x,
  ...,
  project
)

\method{project_exists}{opal}(x, ..., project)

\method{project_exists}{ArmadilloCredentials}(x, ..., project)
}
\arguments{
\item{x}{Connection object to backend for DataSHIELD server (e.g., Opal).}

\item{...}{Optional arguments, unused.}

\item{project}{String with project name to be verified.}
}
\value{
Nothing, call for its side effect. Stop execution of script if
\code{project} does not exist in the given server.
}
\description{
Wrapper for the \code{\link[opalr:opal.project_exists]{opalr::opal.project_exists()}} and
\code{\link[MolgenisArmadillo:armadillo.list_projects]{MolgenisArmadillo::armadillo.list_projects()}} functions.
}
\seealso{
Other Armadillo: 
\code{\link{parse_user_profiles}()}

Other Opal: 
\code{\link{get_project_details}()},
\code{\link{get_project_tables}()},
\code{\link{get_table_permissions}()},
\code{\link{parse_user_profiles}()}

Other Armadillo: 
\code{\link{parse_user_profiles}()}
}
\concept{Armadillo}
\concept{Opal}
\keyword{internal}
